@@ -3,10 +3,10 @@ import argparse
 from mohou.default import create_default_propagator
 from mohou.script_utils import visualize_lstm_propagation
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     parser = argparse.ArgumentParser()
-    parser.add_argument('-pn', type=str, default='kuka_reaching', help='project name')
-    parser.add_argument('-n', type=int, default=150, help='number of visualization')
+    parser.add_argument("-pn", type=str, default="kuka_reaching", help="project name")
+    parser.add_argument("-n", type=int, default=150, help="number of visualization")
 
     args = parser.parse_args()
     project_name = args.pn
